type LogType = 'log' | 'info' | 'warning' | 'error';

interface LogEntry {
  type: LogType;
  message: string;
  timestamp: Date;
  context?: string;
}

class Logger {
  private static instance: Logger;
  private logs: LogEntry[] = [];
  private subscribers: Array<(log: LogEntry) => void> = [];

  private constructor() {}

  public static getInstance(): Logger {
    if (!Logger.instance) {
      Logger.instance = new Logger();
    }
    return Logger.instance;
  }

  private notifySubscribers(log: LogEntry) {
    this.subscribers.forEach(callback => callback(log));
  }

  private addLog(type: LogType, message: string, context?: string) {
    const log: LogEntry = { type, message, timestamp: new Date(), context };
    this.logs.push(log);
    this.notifySubscribers(log);

    if (process.env.NODE_ENV === 'development') {
      const logMessage = context ? `[${context}] ${message}` : message;
      console[type === 'warning' ? 'warn' : type](logMessage);
    }
  }

  public log(message: string, context?: string) {
    this.addLog('log', message, context);
  }

  public info(message: string, context?: string) {
    this.addLog('info', message, context);
  }

  public warning(message: string, context?: string) {
    this.addLog('warning', message, context);
  }

  public error(message: string, context?: string) {
    this.addLog('error', message, context);
  }

  public getLogs(): LogEntry[] {
    return [...this.logs];
  }

  public clear() {
    this.logs = [];
  }

  public subscribe(callback: (log: LogEntry) => void) {
    this.subscribers.push(callback);
    return () => {
      this.subscribers = this.subscribers.filter(sub => sub !== callback);
    };
  }
}

export const logger = Logger.getInstance();
